drop table prescriptions;
drop table patients ;
drop table doctors;
drop table stuffs;
drop table medicines;

create table patients(
	patient_id number(7) , 
	patient_name varchar(12) not null,
	patient_age number(2) check (patient_age>= 15 and patient_age<=80),
	patient_sex varchar(6) not null,
	patient_dept varchar(3) not null,
	patient_type varchar(10) not null,
	patient_residence varchar(10) , 
	primary key(patient_id)
);

create table doctors(
	doctor_id number(5) , 
	doctor_name varchar(20) not null, 
	doctor_age number(2) check(doctor_age>=20 and doctor_age<=70) , 
	doctor_degree varchar(7),
	doctor_room number(3) ,
	doctor_title varchar(10),
	primary key(doctor_id)
);

create table stuffs(
	stuff_id number(5),
	stuff_name varchar(20) not null, 
	stuff_age number(2) , 
	stuff_sex varchar(6) ,
	stuff_position varchar(25) not null,
	primary key(stuff_id)
);

create table medicines(
	medicine_id number (5),
	medicine_name varchar(10) unique,
	medicine_group varchar(15) not null,
	medicine_price number(5) ,
	primary key(medicine_id)
);

create table prescriptions(
	prescription_id number(5) not null,
	prescription_date date,
	patient_id number(7),
	doctor_id number(5) ,
	stuff_id number(5),
	medicine_id number (5),
	foreign key (patient_id)  references patients(patient_id) on delete cascade,
	foreign key (doctor_id)  references doctors(doctor_id) on delete cascade,
	foreign key (stuff_id)  references stuffs(stuff_id) on delete cascade,
	foreign key (medicine_id)  references medicines(medicine_id) on delete cascade
);

describe patients;
describe doctors;
describe stuffs;
describe medicines;
describe prescriptions;

insert into patients values (1107002 , 'Sun' , 22 , 'Male' , 'CSE' , 'Student','LSH');
insert into patients values (1105029 , 'Jonaet' , 23 , 'Male' , 'ME' , 'Student','AEH');
insert into patients values (0702 , 'Dr.Azhar' , 45 , 'Male' , 'CSE' , 'Teacher','R.Area-5');

insert into doctors values (9901 , 'Md. Momin' , 43 ,  'MBBS', 201 , 'Dr');
insert into doctors values (9903 , 'Mrs.Selina' , 35 ,  'FCPS', 203 , 'Prof.');

insert into stuffs values (9801 , 'Md. Fajal' , 50 , 'Male' , 'Head Stuff');
insert into stuffs values (9802 , 'Mrs.Sahina' , 45 , 'Famale' , 'Medicine Provider');
insert into stuffs values (9803 , 'Md.Abul' , 40 , 'Male' , 'Driver');

insert into medicines values(50001 , 'Napa' , 'Paracitamol' ,15);
insert into medicines values(50002 , 'Neutak' , 'Renitidine' ,22);
insert into medicines values(50003, 'Leo' , 'Levofloxacine' ,120);

insert into prescriptions values(101 , '13-AUG-14', 1107002,9901,9802,50003);
insert into prescriptions values(103 , '20-AUG-14', 0702,9903,9803,50001);
insert into prescriptions values(105 , '16-AUG-14', 1107002,9901,9801,50002);

select * from patients ; 
select * from doctors;
select * from stuffs;
select * from medicines;
select * from prescriptions;

alter table patients add phone varchar(11) default '01xxxxxxxxx';
insert into patients values (1107004 , 'Rashik' , 24 , 'Male' , 'CSE' , 'Student','BSMRH','01675013080');
select * from patients ;

select sum(medicine_price) as Total_price from medicines ; 

select patient_id , patient_name from patients where patient_id in (select patient_id  from prescriptions group by patient_id);

select distinct p.patient_name , p.patient_age from patients p join prescriptions using (patient_id);

SET SERVEROUTPUT ON
	DECLARE
		totalForOne medicines.medicine_price%type;
	BEGIN
		select sum(medicine_price) into totalForOne from medicines where medicine_id in (select medicine_id from prescriptions where patient_id =1107002);
		DBMS_OUTPUT.PUT_LINE('Total price for 1107002 is: ' || totalForOne);	 
END;
/
commit;