DROP TABLE history;
DROP TABLE wishlist;
DROP TABLE book;
DROP TABLE category;
DROP TABLE users;
DROP SEQUENCE user_id_seq;
DROP SEQUENCE book_id_seq;

CREATE SEQUENCE book_id_seq;
CREATE SEQUENCE user_id_seq;
	
CREATE TABLE category
(
	cat_id VARCHAR2(5),
	cat_name VARCHAR2(20) NOT NULL,
	PRIMARY KEY(cat_id)
);

CREATE TABLE book
(
	b_id NUMBER(10),
	b_name NVARCHAR2(30) NOT NULL UNIQUE,
	cat_id VARCHAR2(5) NOT NULL,
	publisher VARCHAR2(20),
	author VARCHAR2(20) NOT NULL,
	edition NUMBER(3),
	year_of_pub NUMBER(4) CHECK(year_of_pub>=1000 AND year_of_pub<=9999),
	pages NUMBER(5) NOT NULL,
	free_pages NUMBER(5),
	stock NUMBER(5) CHECK(stock >= 0),
	price NUMBER(10, 2),
	PRIMARY KEY (b_id),
	FOREIGN KEY(cat_id) REFERENCES category(cat_id) ON DELETE CASCADE
);

CREATE OR REPLACE TRIGGER bid_trigger
	BEFORE INSERT ON book
	FOR EACH ROW
BEGIN
	:NEW.b_id := book_id_seq.NEXTVAL;
END;
/

CREATE TABLE users
(
	u_id NUMBER(10),
	u_name VARCHAR2(20) NOT NULL,
	email VARCHAR2(30) NOT NULL UNIQUE,
	dob DATE NOT NULL,
	occupation VARCHAR2(10) NOT NULL,
	country VARCHAR2(15) NOT NULL,
	joining_date DATE NOT NULL,
	PRIMARY KEY (u_id)
);

CREATE OR REPLACE TRIGGER uid_trigger
	BEFORE INSERT ON users
	FOR EACH ROW
BEGIN
	:NEW.u_id := user_id_seq.NEXTVAL;
END;
/

CREATE TABLE history
(
	u_id NUMBER(10),
	b_id NUMBER(10),
	record_date DATE NOT NULL,
	record_type CHAR(1) NOT NULL,
	FOREIGN KEY (u_id) REFERENCES users(u_id) ON DELETE CASCADE,
	FOREIGN KEY (b_id) REFERENCES book(b_id) ON DELETE CASCADE
);

CREATE TABLE wishlist
(
	u_id NUMBER(10),
	b_id NUMBER(10),
	wish_date DATE NOT NULL,
	wish_type CHAR(1) NOT NULL CHECK(wish_type IN ('P','R')),
	FOREIGN KEY (u_id) REFERENCES users(u_id) ON DELETE CASCADE,
	FOREIGN KEY (b_id) REFERENCES book(b_id) ON DELETE CASCADE
);



INSERT INTO category values ('ISL', 'Islamic');
INSERT INTO category values ('PHY', 'Physics');
INSERT INTO category values ('CS', 'Computer Science');
INSERT INTO category values ('LIT', 'Literature');
INSERT INTO category values ('HIS', 'History');
INSERT INTO category values ('MATH', 'Mathematics');




INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, free_pages, stock, price) values ('Database System Concepts', 'CS', 'McGraw Hill', 'Abraham Silberschatz', 5, '2009', 1142, 100, 10, 59);
INSERT INTO book (b_name, cat_id, publisher, author, year_of_pub, pages, free_pages, stock, price) values ('Sherlock Holmes', 'LIT', 'Shikha Publications', 'Arthur Conan Doyle', 1991, 350, 25, 25, 15.5);
INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, stock, price) values ('Differential Equation', 'MATH', 'Prentice Hall', 'Harunur Rashid', 3, 1999, 289, 27, 19);
INSERT INTO book (b_name, cat_id, author, pages, free_pages) values ('Al-Fiqh Al-Akbar', 'ISL','Imam Abu Hanifa(RA)', 20, 20);
INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, free_pages, stock, price) values ('Software Engineering', 'CS', 'McGraw Hill', 'Roger S. Pressman', 5, 2001, 600, 50, 40, 49);
INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, free_pages, stock, price) values ('Beginning ASP.NET 4.5', 'CS', 'John Wiley and Sons', 'Imar Spaanjaars', 1, 2013, 890, 50, 35, 50);
INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, free_pages, stock, price) values ('Hulliday Physics', 'PHY', 'Prentice Hall', 'John Huliday', 4, 2009, 1234, 40, 29, 59);
INSERT INTO book (b_name, cat_id, publisher, author, edition, year_of_pub, pages, free_pages, stock, price) values ('The Lost World', 'LIT', 'Gutenberg', 'Arthur Conan Doyle', 1, 1983, 279, 25, 50, 19);





INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Suhail', 'suhailmahmood@outlook.com', TO_DATE('5-1-91', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Anwar', 'anwar@ymail', TO_DATE('1-1-88', 'DD/MM/YYYY'), 'Engineer', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Iqbal', 'iqbalcmc49@gmail.com', TO_DATE('15-8-89', 'DD/MM/YYYY'), 'Doctor', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Ahmed', 'ahmed@outlook', TO_DATE('6-4-93', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Abduh', 'muhammadabduh@gmail.com', TO_DATE('4-1-95', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Saddam', 'saddamhussain@gmail.com', TO_DATE('11-1-92', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Shah', 'mdanwar@yahoo.com', TO_DATE('15-8-94', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Mohammad', 'mdiqbal@rocketmail.com', TO_DATE('18-7-03', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Suhail', 'suhailmahmood.c070@gmail.com', TO_DATE('5-1-91', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);
INSERT INTO users (u_name, email, dob, occupation, country, joining_date) values('Suhail', 'suhailmahmood@ymail.com', TO_DATE('5-1-91', 'DD/MM/YYYY'), 'Student', 'Bangladesh', SYSDATE);




INSERT INTO history values(1, 8, to_date('1-1-2010', 'DD/MM/YYYY'), 'R');
INSERT INTO history values(2, 5, SYSDATE, 'P');
INSERT INTO history values(6, 7, to_date('3-8-2011', 'DD/MM/YYYY'), 'P');
INSERT INTO history values(10, 3, to_date('3-3-2013', 'DD/MM/YYYY'), 'R');
INSERT INTO history values(5, 1, to_date('9-12-2013', 'DD/MM/YYYY'), 'P');
INSERT INTO history values(7, 2, to_date('3-10-2009', 'DD/MM/YYYY'), 'R');




Prompt ******** ALL BOOKS ********
select b_id, b_name, cat_name, author, price from book join category using(cat_id);

CREATE OR REPLACE FUNCTION get_u_name(user_id users.u_id%TYPE)
	RETURN users.u_name%TYPE IS
	user_name users.u_name%TYPE;
BEGIN
	SELECT u_name into user_name from users where u_id=user_id;
	RETURN user_name;
END;
/

CREATE OR REPLACE FUNCTION get_b_name(book_id book.b_id%TYPE)
	RETURN book.b_name%TYPE IS
	book_name book.b_name%TYPE;
BEGIN
	SELECT b_name into book_name from book where b_id=book_id;
	RETURN book_name;
END;
/

SET SERVEROUTPUT ON;
BEGIN
	DBMS_OUTPUT.PUT_LINE(get_b_name(5));
END;
/

SET SERVEROUTPUT ON;
CREATE OR REPLACE PROCEDURE get_user_rec(user_id users.u_id%TYPE) IS
	CURSOR cur IS
		SELECT b.b_name, b.author, h.record_date, h.record_type from book b join history h using (b_id) where u_id=user_id;
	rec_cur cur%ROWTYPE;
	rec_size number;
BEGIN
	select count(u_id) into rec_size from history where u_id=user_id;
OPEN cur;
	LOOP
		FETCH cur into rec_cur;
		DBMS_OUTPUT.PUT_LINE('***** Record of user: '||get_u_name(user_id)||'*****');
		if rec_cur.record_type='P' then
			DBMS_OUTPUT.PUT_LINE('Book '||cur%ROWCOUNT||': '||rec_cur.b_name||', by '||rec_cur.author||', on '||rec_cur.record_date||' (Purchased)');
		else
			DBMS_OUTPUT.PUT_LINE('Book '||cur%ROWCOUNT||': '||rec_cur.b_name||', by '||rec_cur.author||', on '||rec_cur.record_date||' (Read online)');
		end if;
		EXIT WHEN cur%ROWCOUNT>=rec_size;
	END LOOP;
	CLOSE cur;
END;
/
show errors;

BEGIN
	get_user_rec(10);
END;
/

COMMIT;